//
//  SampleScript.m

#import "SampleScript.h"
#import "Constants.h"

#ifndef kASAppleScriptSuite
#define kASAppleScriptSuite 'ascr'
#endif

#ifndef kASSubroutineEvent
#define kASSubroutineEvent 'psbr'
#endif

#ifndef keyASSubroutineName
#define keyASSubroutineName 'snam'
#endif


@implementation SampleScript

- (void)searchFor:(NSDictionary *)searchDict sender:(id)sender{
	delegate = sender;
	[resultsTitle removeAllObjects];
	//[resultsURL removeAllObjects];
	
	NSString *searchString;
	
	if ([searchDict objectForKey:@"keyword"]) {
		searchString = [searchDict objectForKey:@"keyword"];
	}
	else if ([searchDict objectForKey:@"isbn"] != nil) {
		searchString = [searchDict objectForKey:@"isbn"];
	}
	else if ([searchDict objectForKey:@"title"] != nil) {
		searchString = [searchDict objectForKey:@"title"];
	}
	else {
		[delegate searchReturnedNumberOfResults:0 sender:self];
		return;
	}
	
	// create the first parameter
	NSAppleEventDescriptor* firstParameter =
	[NSAppleEventDescriptor descriptorWithString:searchString];
	
	// create and populate the list of parameters (in our case just one)
	NSAppleEventDescriptor* parameters = [NSAppleEventDescriptor listDescriptor];
	[parameters insertDescriptor:firstParameter atIndex:1];
	
	// create the AppleEvent target
	ProcessSerialNumber psn = {0, kCurrentProcess};
	NSAppleEventDescriptor* target =
	[NSAppleEventDescriptor
	 descriptorWithDescriptorType:typeProcessSerialNumber
	 bytes:&psn
	 length:sizeof(ProcessSerialNumber)];
	
	// create an NSAppleEventDescriptor with the script's method name to call,
	// this is used for the script statement: "on search_for(user_message)"
	// Note that the routine name must be in lower case.
	NSAppleEventDescriptor* handler =
	[NSAppleEventDescriptor descriptorWithString:
	 [@"search_for" lowercaseString]];
	
	// create the event for an AppleScript subroutine,
	// set the method name and the list of parameters
	NSAppleEventDescriptor* event =
	[NSAppleEventDescriptor appleEventWithEventClass:kASAppleScriptSuite
											 eventID:kASSubroutineEvent
									targetDescriptor:target
											returnID:kAutoGenerateReturnID
									   transactionID:kAnyTransactionID];
	[event setParamDescriptor:handler forKeyword:keyASSubroutineName];
	[event setParamDescriptor:parameters forKeyword:keyDirectObject];
	
	// call the event in AppleScript
	NSDictionary* errors = [NSDictionary dictionary];
	NSAppleEventDescriptor *returnDescriptor = [appleScript executeAppleEvent:event error:&errors];
	
	if (returnDescriptor) {
		
		// successful execution
		if (kAENullEvent != [returnDescriptor descriptorType])
		{
			// script returned an AppleScript result
			if (cAEList == [returnDescriptor descriptorType])
			{
				
				//Pull out the list of titles
				int i, numberOfItems = [returnDescriptor numberOfItems];
				for (i = 1; i <= numberOfItems; i++) {
					NSString *nextTitle = [[returnDescriptor descriptorAtIndex:i] stringValue];
					if ([nextTitle isKindOfClass:[NSString class]])
						[resultsTitle addObject:nextTitle];
				}
			}
		}
	}
	else {
		// report any errors from 'errors'
		NSLog(@"Error running script: %@\n%@", errors);
	}
	
	
	[delegate searchReturnedNumberOfResults:[resultsTitle count] sender:self];
	
	
}



- (NSDictionary *)resultNumber:(int)number {

	//Get the details for one of the results and return it as a dictionary
	
	NSMutableDictionary *returnData = [NSMutableDictionary dictionaryWithCapacity:10];
	
	
	
	// create the first parameter
	NSAppleEventDescriptor* firstParameter = [NSAppleEventDescriptor descriptorWithInt32:number];
	
	// create and populate the list of parameters (in our case just one)
	NSAppleEventDescriptor* parameters = [NSAppleEventDescriptor listDescriptor];
	[parameters insertDescriptor:firstParameter atIndex:1];
	
	// create the AppleEvent target
	ProcessSerialNumber psn = {0, kCurrentProcess};
	NSAppleEventDescriptor* target =
	[NSAppleEventDescriptor
	 descriptorWithDescriptorType:typeProcessSerialNumber
	 bytes:&psn
	 length:sizeof(ProcessSerialNumber)];
	
	// create an NSAppleEventDescriptor with the script's method name to call,
	// this is used for the script statement: "on search_for(user_message)"
	// Note that the routine name must be in lower case.
	NSAppleEventDescriptor* handler =
	[NSAppleEventDescriptor descriptorWithString:
	 [@"details_for" lowercaseString]];
	
	// create the event for an AppleScript subroutine,
	// set the method name and the list of parameters
	NSAppleEventDescriptor* event =
	[NSAppleEventDescriptor appleEventWithEventClass:kASAppleScriptSuite
											 eventID:kASSubroutineEvent
									targetDescriptor:target
											returnID:kAutoGenerateReturnID
									   transactionID:kAnyTransactionID];
	[event setParamDescriptor:handler forKeyword:keyASSubroutineName];
	[event setParamDescriptor:parameters forKeyword:keyDirectObject];
	
	// call the event in AppleScript
	NSDictionary* errors = [NSDictionary dictionary];
	NSAppleEventDescriptor *returnDescriptor = [appleScript executeAppleEvent:event error:&errors];
	
	if (returnDescriptor) {
		
		// successful execution
		if (kAENullEvent != [returnDescriptor descriptorType])
		{
			// script returned an AppleScript result
			if (cAEList == [returnDescriptor descriptorType])
			{
				
				//Pull out the list of lists
				int i, numberOfItems = [returnDescriptor numberOfItems];
				for (i = 1; i <= numberOfItems; i++) {
					NSAppleEventDescriptor *dictionaryDesc = [returnDescriptor descriptorAtIndex:i];
					
					//Should be list item with 2 things
					if (cAEList == [dictionaryDesc descriptorType] && [dictionaryDesc numberOfItems] == 2) {
						NSString *key = [[dictionaryDesc descriptorAtIndex:1] stringValue];
						
						/*if ([key isKindOfClass:[NSString class]] {
						 
						 if ([key isEqualToString:@"imageData"]) {
						 
						 }
						 else {
						 */
						
						
						NSString *value = [[dictionaryDesc descriptorAtIndex:2] stringValue];
						
						if ([value isKindOfClass:[NSString class]])
							[returnData setValue:value forKey:key];
						//	}
						//}
					}
				}
			}
		}
	}
	else {
		// report any errors from 'errors'
		NSLog(@"Error running details script: %@\n%@", errors);
	}
	
	
	return returnData;
}

#pragma mark -
#pragma mark Getter

- (NSArray *)resultsTitles {
	return resultsTitle;
}


#pragma mark Init
+ (id)plugin;
{ 
    return [[[self alloc] init] autorelease];
}


- (id)init {
	if (self = [super init]) {
		resultsTitle = [[NSMutableArray array] retain];
		//resultsURL = [[NSMutableArray array] retain];
		
		
		// load the script from a resource by fetching its URL from within our bundle
		NSString* path = [[NSBundle bundleForClass:[self class]] pathForResource:@"Search" ofType:@"scpt"];
		if (path != nil)
		{
			NSURL* url = [NSURL fileURLWithPath:path];
			if (url != nil)
			{
				NSDictionary* errors = [NSDictionary dictionary];
				appleScript = [[NSAppleScript alloc] initWithContentsOfURL:url error:&errors];
				if (appleScript == nil)
				{
					// report any errors from 'errors'
					NSLog(@"Error loading script: %@\n%@", errors);
					
				}
			}
		}
		else {
			NSLog(@"Error: No Search.scpt file found in Bundle.");
		}
		
		
	}
	return self;
}


- (void)dealloc {
	[resultsTitle release];
//	[resultsURL release];
	
	[appleScript release];

	[super dealloc];
}




#pragma mark Connection Delegation

- (void)downloadComplete:(NSString *)HTMLSource {
	// Search the page and create a list of result titles and URLs for the details
	// In this sample we pick all the links that start with "http"
	
	NSScanner *tempScanner = [NSScanner scannerWithString:HTMLSource];
	
	//	[tempScanner scanUpToString:@"<td class=\"tabcontentbgwhite\" valign=\"middle\" nowrap>" intoString:nil];
	//	[tempScanner scanUpToString:@"<a href=\"/dvd-datenbank/" intoString:nil];
	while (![tempScanner isAtEnd]) {
		NSString *theURL = nil;
		NSString *theTitle = nil;
		
		[tempScanner scanUpToString:@"<a href=\"http:" intoString:nil];
		[tempScanner scanUpToString:@"http:" intoString:nil];
		[tempScanner scanUpToString:@"\"" intoString:&theURL];
		[tempScanner scanUpToString:@">" intoString:nil];
		[tempScanner scanUpToString:@"</" intoString:&theTitle];
		
		if (theURL && [theTitle hasPrefix:@">"]) {
			[resultsTitle addObject:[theTitle substringFromIndex:1]];
			//[resultsURL addObject:theURL];
			//[resultsURL addObject:[NSString stringWithFormat:@"http://www.dvd-palace.de/%@", theURL]];
		}
	}
	
	[delegate searchReturnedNumberOfResults:[resultsTitle count] sender:self];
	
}

- (void)downloadWebPage:(NSString *)aWebPage {
		
	//Create request for http query
	NSURLRequest *httpRequest = [NSURLRequest requestWithURL:[NSURL URLWithString:aWebPage] cachePolicy:NSURLRequestUseProtocolCachePolicy timeoutInterval:30.0];
	
	theConnection = [[NSURLConnection alloc] initWithRequest:httpRequest delegate:self];
	
	if (theConnection) {
		receivedData = [[NSMutableData data] retain];
	}
	else {
		[delegate searchReturnedNumberOfResults:0 sender:self];
	}	
}


- (void)downloadWebPage:(NSString *)aWebPage withPost:(NSString *)body encoding:(int)anEncoding {
	//Create request for http query
	NSMutableURLRequest *postRequest = [NSMutableURLRequest requestWithURL:[NSURL URLWithString:aWebPage] cachePolicy:NSURLRequestUseProtocolCachePolicy timeoutInterval:30.0];
	[postRequest setHTTPMethod: @"POST"];
	
	[postRequest setHTTPBody:[body dataUsingEncoding:anEncoding]];
	
	theConnection = [[NSURLConnection alloc] initWithRequest:postRequest delegate:self];
	
	if (theConnection) {
		receivedData = [[NSMutableData data] retain];
	}
	else {
		[delegate searchReturnedNumberOfResults:0 sender:self];
	}	
	
}



/* If there are any redirects this notification is sent and we reset the data to 0  */
- (void)connection:(NSURLConnection *)connection didReceiveResponse:(NSURLResponse *)response
{
	//redirects and such
    [receivedData setLength:0];
}



- (void)connection:(NSURLConnection *)connection didReceiveData:(NSData *)data
{
    // append the new data to the receivedData
    [receivedData appendData:data];
}


- (void)connection:(NSURLConnection *)connection didFailWithError:(NSError *)error
{
    // release the connection, and the data object
    [connection release];
    [receivedData release];
	receivedData = nil;
	
	[delegate searchReturnedNumberOfResults:0 sender:self];
}


-(NSURLRequest *)connection:(NSURLConnection *)connection willSendRequest:(NSURLRequest *)request redirectResponse:(NSURLResponse *)redirectResponse {
	return request;
}


- (void)connectionDidFinishLoading:(NSURLConnection *)connection
{
	//Turn recievedData into a  string try NSUTF encoding if that fails use Latin
	NSString *HTMLSource = [[[NSString alloc] initWithData:receivedData encoding:NSUTF8StringEncoding] autorelease];	
	if (HTMLSource == nil)
		HTMLSource = [[[NSString alloc] initWithData:receivedData encoding:NSISOLatin1StringEncoding] autorelease];	
	
	
    // release the connection, and the data object
    [connection release];
    [receivedData release];
	receivedData = nil;
	
	//All encodings failed return no results otherwise parse
	if (HTMLSource == nil)
		[delegate searchReturnedNumberOfResults:0 sender:self];
	else
		[self downloadComplete:HTMLSource];
	
}


@end
